/* -----------------------------------------------------------------------
 *
 * Project:     ttsnmp
 *
 * Filename:    ttCallBack.c
 *
 * Date:        March 1995
 *
 * Purpose:     Callback handeling.
 *
 * Author:      Bram van der Waaij      
 *
 *           _o______=====____H_=======_H______=====__        __=====_____H_===
 *          |_______|[[[[[     \#######/       [[[[[ o|      |o ]]]]]      \###
 *     ______| [][] |____    B.D. van der Waaij      ||      || 
 *    |o ____|      |__| \___________________________||      ||________________
 *  |_| /DCC |_7189_|   \_|  |  |  |  |  |  |  |  |  ||_|  |_||  |  |  |  |  | 
 * @|_|=|====|======|=o=|=============================|_|@@|_|=|====|======|===
 *  |_| [(o)==(o)]  |__________________|   [(o)==(o)] |_|\/|_| [(o)==(o)]  |___
 * ============================================================================
 ***[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]*/


#include <tcl.h>
#include <tk.h>
#include <unistd.h>
#include <strings.h>
#include <arpa/inet.h>
#include <UT-SNMP.h>
#include <ttsnmp.h>
#include <ttCallList.h>
#include <ttVarBind.h>
#include <ttCallBack.h>


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	Retrieve
 *
 * Arguments:	mReqId, request id (IN).
 *		mErrInd, errorindex (IN). 
 *		mErrTyp, errorstatus (IN).
 *		mVarBinds, variablebindinglist (IN)
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:    	This function print the body of an confirmation.
 *
 * Remarks:	none.
 *
 * -----------------------------------------------------------------------
 */
STATUS Retrieve( AsnInt mReqId, AsnInt mErrInd, AsnInt mErrTyp, AsnVbl *mVarBinds )
{
  TCallInfo callinfo;
  char      paramstr[255];

  if (GetCall(mReqId, &callinfo) == 0)   /* get callinfo for request mReqId */
  {
    DelCall(mReqId);		/* remove call from calllist */

      /* add agent proc name */
    strcat( callinfo.tclcallback, " ");
    strcat( callinfo.tclcallback, callinfo.agentName);

      /* add mReqId : (tcl) integer */
    sprintf(paramstr, " %d", mReqId);
    strcat( callinfo.tclcallback, paramstr);

      /* add mErrTyp : (tcl) eStatus */
    strcat( callinfo.tclcallback, " " );
    strcat( callinfo.tclcallback, SnmpStatusError( mErrTyp ) );

      /* add mErrInd : (tcl) eIndex */
    sprintf(paramstr, " %d", mErrInd);
    strcat( callinfo.tclcallback, paramstr);


      /* add Var bind list : (tcl) vBinds */
    GetVarBindListInfo( callinfo.tclcallback, mVarBinds );

    return Tcl_Eval( ttSnmpInterp, callinfo.tclcallback ); /* exec callback */
  } 
  else return FAILED;
}



/*
 * -----------------------------------------------------------------------
 *
 * Function:    HandleCallBack
 *
 * Arguments:   type, service type (IN).
 *
 * Return:      STATUS, OK or ERROR.
 *
 * Purpose:     This function dispatches the type to the appropriate
 *              confirm apicall.
 *
 * Remarks:     none
 *
 * -----------------------------------------------------------------------
 */
STATUS HandleCallBack( AsnInt mType )
{
  AsnInt mReqId, mInd, mErrTyp;
  AsnVbl *vbl = mVlNew();
  STATUS result = OK;

  mReqId = mInd = mErrTyp = 0; 
  vbl->first=NULL; vbl->last = NULL ; vbl->curr = NULL ; vbl->count = 0 ;

  switch( mType )
  {
    case UTB_GETCF_TYPE:                            /* get command? */
      if( utbGetCf( &mReqId, &mErrTyp, &mInd, vbl ) == OK ) 
        result = Retrieve( mReqId, mInd, mErrTyp, vbl );
      else result = ERROR;
      break;

    case UTB_GETNEXTCF_TYPE:                        /* next command? */
      if( utbGetnextCf( &mReqId, &mErrTyp, &mInd, vbl ) == OK ) 
        result = Retrieve( mReqId, mInd, mErrTyp, vbl );
      else result = ERROR;
      break;

    case UTB_GETBULKCF_TYPE:                        /* bulk command? */
      if( utbGetbulkCf( &mReqId, &mErrTyp, &mInd, vbl ) == OK ) 
        result = Retrieve( mReqId, mInd, mErrTyp, vbl );
      else result = ERROR;
      break;

    case UTB_SETCF_TYPE:                           /* set command? */
      if( utbSetCf( &mReqId, &mErrTyp, &mInd, vbl ) == OK ) 
        result = Retrieve( mReqId, mInd, mErrTyp, vbl );
      else result = ERROR;
      break;

    case UTB_PERROR_TYPE:                          /* error */
      mErrTyp = UTB_PERROR_TYPE ;
      if( utbPerror( &mErrTyp ) == OK ) ;
      result = ERROR;
      break;

    default:
      break;
  }

  mVlFree( vbl );
  return( result );
}

