/* -----------------------------------------------------------------------
 *
 * Project:     ttsnmp
 *
 * Filename:    ttConfig.c
 *
 * Date:        March 1995
 *
 * Purpose:     Config command
 *
 * Author:      Bram van der Waaij      
 *
 *           _o______=====____H_=======_H______=====__        __=====_____H_===
 *          |_______|[[[[[     \#######/       [[[[[ o|      |o ]]]]]      \###
 *     ______| [][] |____    B.D. van der Waaij      ||      || 
 *    |o ____|      |__| \___________________________||      ||________________
 *  |_| /DCC |_7189_|   \_|  |  |  |  |  |  |  |  |  ||_|  |_||  |  |  |  |  | 
 * @|_|=|====|======|=o=|=============================|_|@@|_|=|====|======|===
 *  |_| [(o)==(o)]  |__________________|   [(o)==(o)] |_|\/|_| [(o)==(o)]  |___
 * ============================================================================
 ***[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]*/

#include <tcl.h>
#include <tk.h>
#include <UT-SNMP.h>
#include <ttComm.h>
#include <ttAgentList.h>
#include <ttConfig.h>


void SetConfigResult( Tcl_Interp *interp, AsnOcts *AgentName, TAgentInfo AgentInfo )
{
  AsnOcts *ipaddr;
  char    result[255];

  ipaddr = mOcNew( 255 );
  GetAddr( &ipaddr, &AgentInfo.agentaddr );
  
    /* make result string */
  sprintf(result,"{active %i} {maxrept %i} {name %s} (nonrptr %i } {retries %i} {timeout %i}",
                  1, AgentInfo.maxrept, ipaddr->octs, AgentInfo.nonrptr, 
                  AgentInfo.retries, AgentInfo.timeout );
  
  Tcl_AppendResult( interp, &result[0], (char *) NULL );         /* add result */
}

/*
 * -----------------------------------------------------------------------
 *
 * TCL Command:	<agent> config
 *
 * TCL Args:	<agent> config <item> <value>
 *		<item> : retries, timeout, maxrept, nonrptr
 *		
 * TCL example:	agent config timeout 0
 *
 * Return:	Value (list)
 *
 * Purpose:	Get config information on agent
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
int SnmpAgentConfig( ClientData clientdata, 
                     Tcl_Interp *interp, int argc, char *argv[] ) 
{
  TAgentInfo AgentInfo;
  AsnOcts    *agentname;
  char       result[255];

  MakeAsnOcts( &agentname, argv[0] );                 /* get agent name */
  if (GetAgent( agentname, &AgentInfo ) == FAILED)    /* retrieve agent info */
  {
    Tcl_SetResult( interp, "agent not found", TCL_VOLATILE );
    return TCL_ERROR;
  }

  if (argc == 2)                      /* no arguments, return list */
    SetConfigResult( interp, agentname, AgentInfo );
  
  else
  
    {
      switch(*argv[2]) {
        case 'r':
            if (strcmp(argv[2], "retries") == 0) {
              sprintf( result, "%i", AgentInfo.retries );
              Tcl_AppendResult( interp, &result[0], (char *) NULL );  /* give current value back */
              
              if (argc == 4) {                     /* set retries */
                AgentInfo.retries = atoi( argv[3] );
                SetAgent( agentname, AgentInfo );  /* set AgentInfo */
              }
              break;
            }
     
       case 't':
            if (strcmp(argv[2], "timeout") == 0) {
              sprintf(result,"%i", AgentInfo.timeout );
              Tcl_AppendResult( interp, &result[0], (char *) NULL ); 

              if (argc == 4) {                     /* set timeout */
                AgentInfo.timeout = atoi( argv[3] );
                SetAgent( agentname, AgentInfo );  /* set AgentInfo */
              }
              break;
            }

       case 'm':
            if (strcmp(argv[2], "maxrept") == 0) {
              sprintf(result,"%i", AgentInfo.maxrept );
              Tcl_AppendResult( interp, &result[0], (char *) NULL ); 

              if (argc == 4) {                     /* set maxrept */
                AgentInfo.maxrept = atoi( argv[3] );
                SetAgent( agentname, AgentInfo );  /* set AgentInfo */
              }
              break;
            }

       case 'n':
            if (strcmp(argv[2], "nonrptr") == 0) {
              sprintf(result,"%i", AgentInfo.nonrptr );
              Tcl_AppendResult( interp, &result[0], (char *) NULL ); 

              if (argc == 4) {                     /* set nonrptr */
                AgentInfo.nonrptr = atoi( argv[3] );
                SetAgent( agentname, AgentInfo );  /* set AgentInfo */
              }
              break;
            }

        default:
            Tcl_SetResult( interp, "Unknown option to config", TCL_VOLATILE );
            return TCL_ERROR; 

      }
    }

  return TCL_OK;
}
