/* -----------------------------------------------------------------------
 *
 * Project:     ttsnmp
 *
 * Filename:    ttVarBind.c
 *
 * Date:        March 1995
 *
 * Purpose:     Varbind functions
 *
 * Author:      Bram van der Waaij      
 *
 *           _o______=====____H_=======_H______=====__        __=====_____H_===
 *          |_______|[[[[[     \#######/       [[[[[ o|      |o ]]]]]      \###
 *     ______| [][] |____    B.D. van der Waaij      ||      || 
 *    |o ____|      |__| \___________________________||      ||________________
 *  |_| /DCC |_7189_|   \_|  |  |  |  |  |  |  |  |  ||_|  |_||  |  |  |  |  | 
 * @|_|=|====|======|=o=|=============================|_|@@|_|=|====|======|===
 *  |_| [(o)==(o)]  |__________________|   [(o)==(o)] |_|\/|_| [(o)==(o)]  |___
 * ============================================================================
 ***[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]*/

#include <tcl.h>
#include <tk.h>
#include <UT-SNMP.h>
#include <ttVarBind.h>

  /* define for use in AddWriteVarList function */
#define mVb(n,c,v) \
           mVbPut(mVbNew(255),n,mVcPut(mVcNew(),VARBINDCHOICE_VALUE,\
           mObPut(mObNew(),c,v)))


/*
 * -----------------------------------------------------------------------
 *
 * Function:	GetObjectSyntax
 *
 * Arguments:	tclcommand, string with the Object Syntax (OUT)
 *		value, pointer to retrieved ObjectSyntax structure (IN)
 *
 * Return:	void
 *
 * Purpose:	transform ObjectSyntax struct to string
 *
 * Remarks:	problems with:
 *		  bitstring 
 *		  non printable strings (OctetString) 
 *		  NsapAddress
 *
 * -----------------------------------------------------------------------
 */
void GetObjectSyntax(char *tclcommand,  struct ObjectSyntax *value)
{
  char    str[255];
  AsnOcts *asnstr;

  asnstr = mOcNew( 255 );
  strcat(tclcommand, " ");  /* basic space char before parameter */

  switch( value->choiceId )
  {
    case(OBJECTSYNTAX_INTEGER_VALUE):
      strcat( tclcommand, "Integer32 " ); 
      sprintf(str,"%i", value->a.integer_value);
      strcat( tclcommand, str );
      break;

    case(OBJECTSYNTAX_STRING_VALUE):
      strcat( tclcommand, "DisplayString ");
      strcat( tclcommand ,"{" );
      strncat( tclcommand, value->a.string_value->octs, 
                           value->a.string_value->octetLen );
      strcat( tclcommand, "}" );
      break;

    case(OBJECTSYNTAX_OBJECTID_VALUE):
      strcat( tclcommand, "ObjectID " );
      OIDtoASCII( (AsnOid *) value->a.objectID_value, str );
      strcat( tclcommand, str);
      break;

    case(OBJECTSYNTAX_BIT_VALUE):
      strcat( tclcommand, "BitString " );
      strncat( tclcommand, value->a.bit_value->bits,
                           value->a.bit_value->bitLen );
      break;

    case(OBJECTSYNTAX_IPADDRESS_VALUE):
      strcat( tclcommand, "IpAddress " );
      GetAddr( &asnstr, value->a.ipAddress_value );
      strncat( tclcommand, asnstr->octs,
                           asnstr->octetLen );
      break;

    case(OBJECTSYNTAX_COUNTER_VALUE):
      strcat( tclcommand, "Counter32 " );
      sprintf(str,"%i",value->a.counter_value);
      strcat( tclcommand, str );
      break;

    case(OBJECTSYNTAX_GAUGE_VALUE):
      strcat( tclcommand, "Gauge32 " );
      sprintf(str,"%i",value->a.gauge_value);
      strcat( tclcommand, str );
      break;

    case(OBJECTSYNTAX_TIMETICKS_VALUE):
      strcat( tclcommand, "TimeTicks " );
      sprintf(str,"%i",value->a.timeticks_value);
      strcat( tclcommand, str );
      break;

    case(OBJECTSYNTAX_ARBITRARY_VALUE):
      strcat( tclcommand, "Arbitrary " );
      strncat( tclcommand, value->a.arbitrary_value->octs,
                           value->a.arbitrary_value->octetLen );
      break;

    case(OBJECTSYNTAX_NSAPADDRESS_VALUE):
      strcat( tclcommand, "NsapAddress " );
      strncat( tclcommand, value->a.nsapAddress_value->octs,
                           value->a.nsapAddress_value->octetLen );
      break;

    case(OBJECTSYNTAX_BIG_COUNTER_VALUE):
      strcat( tclcommand, "Counter64 " );
      sprintf(str,"%i",value->a.big_counter_value);
      strcat( tclcommand, str );
      break;

    case(OBJECTSYNTAX_UNSIGNED_INTEGER_VALUE):
      strcat( tclcommand, "UInteger32 ");
      sprintf(str,"%i",value->a.unsigned_integer_value);
      strcat( tclcommand, str );
      break;

  }
}

/*
 * -----------------------------------------------------------------------
 *
 * Function:	GetVarBind
 *
 * Arguments:	tclcommand, string with the Object Syntax (OUT)
 *		VarBind, retrieved VarBind
 *
 * Return:	void
 *
 * Purpose:	Translate one VarBind into a tcl handleble string
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
void GetVarBind( char *tclcommand,  VarBind *vb) 
{
  char oidstr[255];

  strcat(tclcommand, "{" );

  OIDtoASCII( (AsnOid *) &vb->name, oidstr );
  strcat( tclcommand, oidstr);

  switch( vb->varBindChoice->choiceId )
  {
      case(VARBINDCHOICE_VALUE):
          GetObjectSyntax( tclcommand, vb->varBindChoice->a.value );
          break;

      case(VARBINDCHOICE_UNSPECIFIED):
        strcat( tclcommand, " execption unSpecified " );

      case(VARBINDCHOICE_NOSUCHOBJECT):
        strcat( tclcommand, " exception noSuchObject " );
        break;

      case(VARBINDCHOICE_NOSUCHINSTANCE):
        strcat( tclcommand, " exception noSuchInstance " );
        break;

      case(VARBINDCHOICE_ENDOFMIBVIEW):
        strcat( tclcommand, " exception endOfMibView " );
        break;
  }

  strcat(tclcommand, "} " );
}

/*
 * -----------------------------------------------------------------------
 *
 * Function:	GetVarBindListInfo
 *
 * Arguments:	tclcommand, string with the Object Syntax (OUT)
 *		VarBindList, Retrieved list of varbinds
 *
 * Return:	OK
 *
 * Purpose:	Translate the varbindlist to a tcl string
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
STATUS GetVarBindListInfo( char *tclcommand, VarBindList *vbl )
{
  VarBind *vb;

    /* open tcl list arguments */
  strcat(tclcommand, " {" );

  if (vbl != NULL) 
    FOR_EACH_LIST_ELMT( vb, vbl ) 
      GetVarBind( tclcommand, vb );
    
    /* close tcl list arguments */
  strcat(tclcommand, "}" );

  return OK;
}

/*
 * -----------------------------------------------------------------------
 *
 * Function:	SyntaxToChoice
 *
 * Arguments:	Syntax: pointer to Syntax description (IN )
 *		Choice return value (OUT)
 *
 * Return:	OK 
 *
 * Purpose:	Transform user input into ObjectSyntaxChoiceId
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
STATUS SyntaxToChoice(ObjectSyntaxChoiceId *Choice, char *Syntax ) 
{
  /* look which Choice is made */
  switch( Syntax[0] )
  {
    case 'I':
        if (strcmp( Syntax, "Integer32") == 0) {
          *Choice = OBJECTSYNTAX_INTEGER_VALUE;
          break;
        }

       if (strcmp( Syntax, "IpAddress") == 0) {
          *Choice = OBJECTSYNTAX_IPADDRESS_VALUE;
          break;
        }

    case 'D':
        if (strcmp( Syntax, "DisplayString") == 0) {
          *Choice = OBJECTSYNTAX_STRING_VALUE;
          break;
        }

    case 'O':
        if (strcmp( Syntax, "ObjectID") == 0) {
          *Choice = OBJECTSYNTAX_OBJECTID_VALUE;
          break;
        }

    case 'B':
        if (strcmp( Syntax, "BitString") == 0) {
          *Choice = OBJECTSYNTAX_BIT_VALUE;
          break;
        }
  
    case 'C':
        if (strcmp( Syntax, "Counter32") == 0) {
          *Choice = OBJECTSYNTAX_COUNTER_VALUE;
          break;
        }
      
       if (strcmp( Syntax, "Counter64") == 0) {
          *Choice = OBJECTSYNTAX_BIG_COUNTER_VALUE;
          break;
        }
 
    case 'G':
        if (strcmp( Syntax, "Gauge32") == 0) {
          *Choice = OBJECTSYNTAX_GAUGE_VALUE;
          break;
        }

    case 'T':
        if (strcmp( Syntax, "TimeTicks") == 0) {
          *Choice = OBJECTSYNTAX_TIMETICKS_VALUE;
          break;
        }

    case 'A':
        if (strcmp( Syntax, "Arbitrary") == 0) {
          *Choice = OBJECTSYNTAX_ARBITRARY_VALUE;
          break;
        }

    case 'N':
        if (strcmp( Syntax, "NsapAddress") == 0) {
          *Choice = OBJECTSYNTAX_NSAPADDRESS_VALUE;
          break;
        }

     case 'U':
        if (strcmp( Syntax, "UInteger32") == 0) {
          *Choice = OBJECTSYNTAX_UNSIGNED_INTEGER_VALUE;
          break;
        }
     
     default:
       *Choice = OBJECTSYNTAX_ARBITRARY_VALUE;
       break;
  }

  return OK;
}



/*
 * -----------------------------------------------------------------------
 *
 * Function:	AddReadVarList
 *
 * Arguments:	interp, TCL interpreter pointer (IN)
 *		OIDList, pointer to OID list argument in commandline (IN)
 *		vbl, pointer to existing var bind list (OUT)
 *
 * Return:	OK or FAILED
 *
 * Purpose:	Fill the vbl with OID
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
STATUS AddReadVarList( Tcl_Interp *interp, 
                       char *OIDList,
                       AsnVbl *vbl )
{
  int listArgc;
  char **listArgv;
  AsnOid *oid;
  VarBind *vb;
  int i;

    /* abstract oid list from commandline arguments */
  if (Tcl_SplitList(interp, OIDList, &listArgc, &listArgv) != TCL_OK)
    return FAILED;

  for (i=0; i<listArgc; i++) {  /* for each oid-element do: */
  
    ASCIItoOID( listArgv[i], oid = mOiNew( 255) );   /* create oid */

       /* create varbind: vb */
    vb = mVbPut( mVbNew(255), oid, 
                 mVcPut( mVcNew( ), VARBINDCHOICE_UNSPECIFIED, 0 ) ) ;
    mVlPut( vbl, vb );                              /* put vb in vbl */
  
  }

  free ((char *) listArgv);
  return( OK );
}


STATUS AddSetVarBind(char *OID, 
                     char *Syntax, 
                     char *Value, 
                     AsnVbl *vbl)
{
  AsnOid *oid, *oidValue;
  VarBind *vb;
  ObjectSyntaxChoiceId Choice;

  ASCIItoOID( OID, oid = mOiNew( 255) ) ;	/* create oid */

  if (SyntaxToChoice( &Choice, Syntax ) != OK ) 
    return FAILED;

  switch( Choice )
  {
     case(OBJECTSYNTAX_INTEGER_VALUE):
       vb = mVb( oid, OBJECTSYNTAX_INTEGER_VALUE, 
                 atoi( Value ) );
            break;

         case(OBJECTSYNTAX_STRING_VALUE):
            vb = mVb( oid, OBJECTSYNTAX_STRING_VALUE, 
                 mOcPut( mOcNew(255), Value, strlen(Value)) );
            break;

         case(OBJECTSYNTAX_OBJECTID_VALUE):
            ASCIItoOID( Value, oidValue = mOiNew( 255 ) );
            vb = mVb( oid, OBJECTSYNTAX_OBJECTID_VALUE, oidValue );
            break;

        case(OBJECTSYNTAX_COUNTER_VALUE):
            vb = mVb( oid, OBJECTSYNTAX_COUNTER_VALUE,
                      atoi( Value ) );
            break;

        case(OBJECTSYNTAX_GAUGE_VALUE):
            vb = mVb( oid, OBJECTSYNTAX_GAUGE_VALUE,
                      atoi( Value ) );
            break;

        case(OBJECTSYNTAX_TIMETICKS_VALUE):
            vb = mVb( oid, OBJECTSYNTAX_TIMETICKS_VALUE,
                      atoi( Value ) );
            break;
 
        case(OBJECTSYNTAX_ARBITRARY_VALUE):
            break;

        case(OBJECTSYNTAX_NSAPADDRESS_VALUE):
            break;

        case(OBJECTSYNTAX_BIG_COUNTER_VALUE): 
            vb = mVb( oid, OBJECTSYNTAX_BIG_COUNTER_VALUE,
                      atoi( Value ) );
            break;

        case(OBJECTSYNTAX_UNSIGNED_INTEGER_VALUE):
            vb = mVb( oid, OBJECTSYNTAX_UNSIGNED_INTEGER_VALUE, 
                      atoi( Value ) );
            break;
      }  /* switch */

    mVlPut( vbl, vb ) ;				/* put vb in vbl */

  return OK;
}


/*
 * -----------------------------------------------------------------------
 *
 * Function:	AddWriteVarList
 *
 * Arguments:	vbl, pointer to existing var bind list (OUT)
 *		OID, oid to add with vbl (IN)
 *		Choice, Syntax choice 
 *		Value, Value to be set 
 *
 * Return:	OK
 *
 * Purpose:	Fill the vbl with OID, Choice and value
 *
 * Remarks:	Arbitrary Value, nSapAddress 
 *		BIT, IpAddress not yet implemented !
 *
 * -----------------------------------------------------------------------
 */
STATUS AddWriteVarList( Tcl_Interp *interp,
                        char *SetList,
                        AsnVbl *vbl )
{
  int  SetListArgc, CallListArgc;
  char **SetListArgv, **CallListArgv;
  int i;

    /* abstract element list from commandline arguments */
  if (Tcl_SplitList(interp, SetList, &SetListArgc, &SetListArgv) != TCL_OK)
    return FAILED;

    /* no space character in first listelement -> only one set operation
      is wanted. If there is a space-char in the list, each SetListArgv
      contains a complete set operation */
  if (strchr(SetListArgv[0], (int) ' ') == NULL) {  /* no space */ 

    if (SetListArgc != 3) {  /* wrong set of elements */
      fprintf(stderr,"Wrong set of elements\n");
      return FAILED;
    }
               
    AddSetVarBind(SetListArgv[0], SetListArgv[1], SetListArgv[2], vbl );
  }

  else {
    for (i=0; i<SetListArgc; i++) {  /* for each set-element do: */

        /* abstract set list from commandline arguments */
      if (Tcl_SplitList(interp, SetListArgv[i],
                        &CallListArgc, &CallListArgv) != TCL_OK)
        return FAILED;

      if (CallListArgc != 3) {  /* wrong set of elements */
        fprintf(stderr,"Wrong set of elements\n");
        return FAILED;
      }

      AddSetVarBind(CallListArgv[0], CallListArgv[1], CallListArgv[2], vbl); 
      
      free ((char *) CallListArgv);
    }  /* for i */
  }

  free ((char *) SetListArgv);
  return OK ;
}
