/* -----------------------------------------------------------------------
 *
 * Project:     ttsnmp
 *
 * Filename:    ttsnmp.c
 *
 * Date:        March 1995
 *
 * Purpose:     main part
 *
 * Author:      Bram van der Waaij      
 *
 *           _o______=====____H_=======_H______=====__        __=====_____H_===
 *          |_______|[[[[[     \#######/       [[[[[ o|      |o ]]]]]      \###
 *     ______| [][] |____    B.D. van der Waaij      ||      || 
 *    |o ____|      |__| \___________________________||      ||________________
 *  |_| /DCC |_7189_|   \_|  |  |  |  |  |  |  |  |  ||_|  |_||  |  |  |  |  | 
 * @|_|=|====|======|=o=|=============================|_|@@|_|=|====|======|===
 *  |_| [(o)==(o)]  |__________________|   [(o)==(o)] |_|\/|_| [(o)==(o)]  |___
 * ============================================================================
 ***[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]*/



/**********
 *
 * This program has no main part -> tcl has
 *
 **********/

#include <tcl.h>
#include <tk.h>
#include <UT-SNMP.h>
#include <ut-foid.h>
#include <ttComm.h>
#include <ttAgentList.h>
#include <ttCallList.h>
#include <ttNewSnmp.h>
#include <ttCallBack.h>
#include <ttDebug.h>
#include <ttTimer.h>
#include <ttFoid.h>
#include <ttsnmp.h>



/*
 * -----------------------------------------------------------------------
 *
 * Function:    InitSystem
 *
 * Arguments:   none
 *
 * Return:      none
 *
 * Purpose:     Make connection with ut-spm
 *		Create Agent list
 *		Create Call list
 *
 * Remarks:	none
 *
 * -----------------------------------------------------------------------
 */
void InitSystem()
{
  AsnOcts Appname;
  char    rnd[30];
  
    /* create random name to connect to the ut-spm */
  sprintf(rnd, "%i", random() );
  Appname.octs = malloc( 30 );
  strcpy(Appname.octs, "ttsnmp");
  strncat(Appname.octs, rnd, 5 );
  Appname.octetLen = strlen(Appname.octs);

  if (MakeManagerConn( &Appname ) != OK)  /* connect to agent */
  {
    fprintf(stderr,"Error during connection to ut-spm, system halting\n");
    exit(-1);
  }

  if (Foid_Init() != OK) 
    fprintf(stderr,"void database not loaded!\n");

  CreateAgentList();
  CreateCallList();

  fprintf(stderr,"%s: type snmp for more information\n",Appname.octs);
}


/*
 * -----------------------------------------------------------------------
 *
 * TCL Command:	exit
 *
 * TCL Args:	none
 *
 * TCL example:	exit
 *
 * Return:	none
 *
 * Purpose:	Close connection with ut-spm
 *		Remove Agent list
 *		Remove Call list
 *
 * Remarks:	none
 *
 * -----------------------------------------------------------------------
 */
int ExitSystem( ClientData clientdata, Tcl_Interp *interp, int argc, char *argv[] )
{
  if (CloseConnection() != OK)
    fprintf(stderr,"Error during closing from ut-spm\n");

  if (Foid_Exit() != OK)
    fprintf(stderr,"void database not properly closed\n");

  RemoveAgentList();
  RemoveCallList();

  exit(0);
}



/*
 * -----------------------------------------------------------------------
 *
 * Function:    CreateCommands
 *
 * Arguments:   interp, Tcl interpreter pointer
 *
 * Return:      void
 *
 * Purpose:     Create the basic ttsnmp commands:
 *		exit, newsnmp, listsnmp, helpsnmp, timersnmp
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
void CreateCommands( Tcl_Interp *interp )
{
  Tcl_CreateCommand( interp, "exit", ExitSystem, 
                     (ClientData *) NULL,  (Tcl_CmdDeleteProc *) NULL );

  Tcl_CreateCommand( interp, "newsnmp", NewSnmpCommand, 
                     (ClientData *) NULL,  (Tcl_CmdDeleteProc *) NULL );

  Tcl_CreateCommand( interp, "snmp", SnmpCommand, 
                     (ClientData *) NULL,  (Tcl_CmdDeleteProc *) NULL );

  Tcl_CreateCommand( interp, "foidtooid", FoidToOIDCommand, 
                     (ClientData *) NULL,  (Tcl_CmdDeleteProc *) NULL );
  
  Tcl_CreateCommand( interp, "oidtofoid", OIDToFoidCommand, 
                     (ClientData *) NULL,  (Tcl_CmdDeleteProc *) NULL );

}


/*
 * -----------------------------------------------------------------------
 *
 * Function:    Tcl_Appinit
 *
 * Arguments:   interp, Tcl interpreter pointer
 *
 * Return:      
 *
 * Purpose:     Init the application
 *
 * Remarks:	Redefinition of a tcl function !
 *
 * -----------------------------------------------------------------------
 */
int Tcl_AppInit( Tcl_Interp *interp )
{
  Tk_Window   main; 

  main = Tk_MainWindow(interp);
  if (main == NULL) {
      return TCL_ERROR;
  }

  if (Tcl_Init(interp) == TCL_ERROR) 
      return TCL_ERROR;

    /* This call tot InitSystem MUST be between Tcl_Init and Tk_Init
       otherwise the ut-spm crashes, I don't know why ;-) */
  InitSystem();  /* make connection with agent */


  if (Tk_Init(interp) == TCL_ERROR) 
      return TCL_ERROR;
 
  CreateCommands( interp );

    /* create callback checking timer */ 
  SetPollTimer();
  SetOutTimer();

  ttSnmpInterp = interp;

  return( TCL_OK );
}
